package com.bomostory.sceneeditmodule.cover

import android.content.Context
import android.graphics.Bitmap
import android.support.constraint.ConstraintLayout
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.RadioButton
import com.bomostory.sceneeditmodule.SceneDrawer
import com.bomostory.sceneeditmodule.basicdata.Project
import com.bomostory.sceneeditmodule.basicdata.Scene
import com.bomostory.sceneeditmodule.utils.Utils
import com.example.tfat.myapplication.R
import kotlinx.android.synthetic.main.item_front_cover_scene_chooser.view.*
import kotlinx.android.synthetic.main.view_front_cover_scene_chooser.view.*

class FrontCoverSceneChooserView @JvmOverloads constructor(
        context: Context,
        attrs: AttributeSet? = null,
        defStyleAttr: Int = 0
) : ConstraintLayout(context, attrs, defStyleAttr) {

    var selected: Int = 0
        private set
    private var project: Project? = null
        set(value) {
            field = value
            update()
        }
    var title: String = ""
        set(value) {
            tv_frontCoverSceneChooserDialog_coverTitle.text = value
        }
    var author: String = ""
        set(value) {
            tv_frontCoverSceneChooserDialog_coverAuthor.text = value
        }
    var onApply: View.OnClickListener? = null
    var onDiscard: View.OnClickListener? = null

    private val dataList = ArrayList<Data>()
    private val adapter = Adapter(dataList)
    private var lastSelectedData: Data? = null

    init {
        inflate(context, R.layout.view_front_cover_scene_chooser, this)
        setOnClickListener {
            // do nothing
        }
        btn_frontCoverSceneChooserDialog_apply.setOnClickListener { onApply?.onClick(it) }
        btn_frontCoverSceneChooserDialog_discard.setOnClickListener { onDiscard?.onClick(it) }
        recyclerView_frontCoverSceneChooserDialog_sceneList.layoutManager = LinearLayoutManager(context).apply {
            orientation = LinearLayoutManager.VERTICAL
        }
        recyclerView_frontCoverSceneChooserDialog_sceneList.adapter = adapter
    }

    fun setProjectAndSelected(project: Project, selected: Int) {
        this.selected = selected
        this.project = project
    }

    private fun update() {
        val p = project ?: return
        dataList.clear()
        var i = 0
        p.story?.scenes?.forEach { scene ->
            val width = context.resources.getDimension(R.dimen.share_dialog_screenshot_width).toInt()
            val height = context.resources.getDimension(R.dimen.share_dialog_screenshot_height).toInt()
            val bitmap = SceneDrawer.drawScene(context, scene, 0, width, height) ?: return@forEach
            val isSelected = i == selected
            val name = Utils.generateSceneName(i)
            val onSelectedListener = object : OnSelectedListener {
                override fun onSelected(data: Data) {
                    lastSelectedData?.isSelected = false
                    lastSelectedData = data
                    lastSelectedData?.isSelected = true
                    project?.story?.scenes?.also { scenes ->
                        selected = dataList.indexOf(data)
                        val s = scenes[selected]
                        onSceneSelected(s)
                    }
                }
            }
            val data = Data(isSelected, bitmap, name, onSelectedListener)
            dataList.add(data)
            if (isSelected) {
                onSceneSelected(scene)
            }
            i++
        }
        adapter.notifyDataSetChanged()
    }

    private fun onSceneSelected(scene: Scene) {
        val width = context.resources.getDimension(R.dimen.share_dialog_screenshot_width).toInt()
        val height = context.resources.getDimension(R.dimen.share_dialog_screenshot_height).toInt()
        val bitmap = SceneDrawer.drawScene(context, scene, 0, width, height) ?: return
        iv_frontCoverSceneChooserDialog_cover.setImageBitmap(bitmap)
    }

    private inner class Adapter(private val dataList: List<Data>) : RecyclerView.Adapter<ViewHolder>() {

        private var lastCheckedRadioButton: RadioButton? = null

        private val onSelectedListener = object : ViewHolder.OnSelectedListener {
            override fun onSelected(radioButton: RadioButton) {
                lastCheckedRadioButton?.isChecked = false
                lastCheckedRadioButton = radioButton
                lastCheckedRadioButton?.isChecked = true
            }
        }

        override fun onCreateViewHolder(parent: ViewGroup, type: Int): ViewHolder {
            val v1 = LayoutInflater.from(parent.context).inflate(R.layout.item_front_cover_scene_chooser, parent, false)
            return ViewHolder(v1)
        }

        override fun getItemCount(): Int {
            return dataList.size
        }

        override fun onBindViewHolder(p0: ViewHolder, p1: Int) {
            val data = dataList[p1]
            p0.bind(data, onSelectedListener)
        }

        override fun onViewRecycled(holder: ViewHolder) {
            super.onViewRecycled(holder)
            holder.itemView.rb_frontCoverSceneChooserItem.setOnCheckedChangeListener(null)
        }
    }

    private class ViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        fun bind(data: Data, onSelectedListener: OnSelectedListener) {
            if (data.isSelected) {
                onSelectedListener.onSelected(itemView.rb_frontCoverSceneChooserItem)
            }
            itemView.rb_frontCoverSceneChooserItem.setOnCheckedChangeListener { _, b ->
                if (b) {
                    onSelectedListener.onSelected(itemView.rb_frontCoverSceneChooserItem)
                    data.onSelectedListener.onSelected(data)
                }
            }
            itemView.iv_frontCoverSceneChooserItem_screenShot.setImageBitmap(data.screenshot)
            itemView.tv_frontCoverSceneChooserItem_name.text = data.name
            itemView.setOnClickListener {
                itemView.rb_frontCoverSceneChooserItem.isChecked = true
            }
            itemView.rb_frontCoverSceneChooserItem.isChecked = data.isSelected
        }

        interface OnSelectedListener {
            fun onSelected(radioButton: RadioButton)
        }
    }

    private data class Data(
            var isSelected: Boolean,
            val screenshot: Bitmap,
            val name: String,
            val onSelectedListener: OnSelectedListener
    )

    private interface OnSelectedListener {
        fun onSelected(data: Data)
    }
}